function [a, X_new] = cis_as2x(w, x, X, xp, model)
% Gaussian model collapse
% w = weights at t-1 [1 N]
% x = particles at t-1 [nx N]
% X = reference trajectory (collapsed) [1 ell]
% xp = reference particle (non-collapsed), [nx 1]

% Extract the model
A = model.A;
mu=model.mu;
Q = model.Q;
M=model.Nx;
ell = size(X,1);
[nx,N] = size(x);

% Simulate X from p(X|X_t-1, X_{t+ell})
% if(ell == nx-1) % Deterministic setting
%     Pxx = 0;
%     x_now = x;
%     for(t = 1:nx) % Collapse over 1...ell, p(x_{t+\ell} | x_{t-1})
%         x_pred = A(1:M,:)*x_now;
%         Pxx = Pxx + loggausspdf2(xp(t)-x_pred, Q(1,1));%loggausspdf2(xp(t)-x_pred, 1/Q(1,1));
%         x_now = [xp(t)*ones(1,N) ; x_now(1:end-1,:)];
%     end
%     
%    X_new = X;
% 
%     % Ancestor sampling
%     Wght = exp(Pxx-max(Pxx));
%     Wght = Wght.*w;
%     Wght = Wght/sum(Wght);
%     a = find(rand(1) < cumsum(Wght), 1,'first');
% else
    % Store filtered estimates for BSi
    x_filt = zeros(nx, N, ell); % We collapse over ell steps
    P_filt = zeros(nx, nx, ell);
    x_filt(:,:,1) = A*x+mu';
    P_filt(:,:,1) = Q;
    
    % Pure prediction forward
    if ell>1
    for(t = 2:ell)
        x_filt(:,:,t) = A*x_filt(:,:,t-1)+mu';
        P_filt(:,:,t) = A*P_filt(:,:,t-1)*A' + Q;
    end
    end
    % Condition on xp (if this is available)
    C = A;
    R = Q;
    xp1=(xp(end:-1:1,1:M));
    xp1=vecr(xp1(1:M,:));
   
%     innovation = bsxfun(@minus, xp(end:-1:1), C*x_filt(:,:,ell));
 innovation = bsxfun(@minus, xp1, mu'+C*x_filt(:,:,ell));
    S = C * P_filt(:,:,ell) * C' + R;
    % K = P_pred * C' * inv(S);
    K = (P_filt(:,:,ell) * C') / S;
    x_filt(:,:,ell) = x_filt(:,:,ell) + K*innovation;
%     P_filt(:,:,ell) = P_filt(:,:,ell) - K*C*P_filt(:,:,ell);
    % Likelihood
    Pxx = loggausspdf2(innovation, S);
% Ancestor sampling
    Wght = exp(Pxx-max(Pxx));
    Wght = Wght.*w;
    Wght = Wght/sum(Wght);
    a = find(rand(1) < cumsum(Wght), 1,'first');
    X_new=x_filt(:,a);
end



